export var isBrowser = typeof global.window === 'undefined' ? false : true;
export function isOnline() {
    return isBrowser ? global.navigator.onLine : true;
}
export function isVisible() {
    return isBrowser ? global.document.visibilityState === 'visible' : true;
}
export function isURL(str) {
    var regex = '^' +
        '(?:wss|ws)://' +
        '[^\\s]+' +
        '(?::\\d{2,5})?' +
        '$';
    return new RegExp(regex, 'i').test(str);
}
export function generateKey() {
    var mask = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    var length = 42;
    var values = randNumbers(length);
    var result = '';
    for (var i = 0; i < length; i++) {
        result += mask[values[i] % mask.length];
    }
    return result;
}
export function generateId(exclude) {
    if (exclude === void 0) { exclude = []; }
    var id = randNumbers()[0];
    if (exclude.includes(id)) {
        return generateId(exclude);
    }
    return id;
}
var MAX_KEY_LENGTH = 512;
export function validateKey(key) {
    if (typeof key !== 'string') {
        throw new Error("The key type \"" + typeof key + "\" is not a \"string\"");
    }
    else if (key === '') {
        throw new Error('The key is an empty string');
    }
    else if (key.length > MAX_KEY_LENGTH) {
        throw new Error("The key length of " + key.length + " exceeds the maximum of " + MAX_KEY_LENGTH + " characters");
    }
    return true;
}
export function extractHostnameAndPort(url) {
    return url.split('/')[2];
}
function randNumbers(length) {
    if (length === void 0) { length = 1; }
    var res;
    if (isBrowser) {
        res = new Uint32Array(length);
        global.crypto.getRandomValues(res);
    }
    else {
        res = [];
        var bytes = global.cryptoNode.randomBytes(4 * length);
        for (var i = 0; i < bytes.length; i += 4) {
            res[res.length] = bytes.readUInt32BE(i, true);
        }
    }
    return res;
}
export function equal(array1, array2) {
    return (array1 !== undefined &&
        array2 !== undefined &&
        array1.length === array2.length &&
        array1.every(function (v) { return array2.includes(v); }));
}
export function isWebSocketSupported() {
    return !!global.WebSocket;
}
export function isWebRTCSupported() {
    return !!global.RTCPeerConnection && 'createDataChannel' in global.RTCPeerConnection.prototype;
}
export * from './util.log';
//# sourceMappingURL=util.js.map