var PendingRequests = (function () {
    function PendingRequests(wcStreamId) {
        this.wcStreamId = wcStreamId;
        this.wcReqs = {
            connectReqs: new Map(),
            pingReqs: new Map(),
        };
        this.sigReqs = {
            connectReqs: new Map(),
            pingReqs: new Map(),
        };
    }
    PendingRequests.prototype.add = function (streamId, id, timeout) {
        var reqs = this.getReqsByStreamId(streamId);
        return this.addRequest(reqs.connectReqs, id, timeout);
    };
    PendingRequests.prototype.addPing = function (streamId, id, timeout) {
        var reqs = this.getReqsByStreamId(streamId);
        return this.addRequest(reqs.pingReqs, id, timeout);
    };
    PendingRequests.prototype.get = function (streamId, id) {
        var req = this.getReqsByStreamId(streamId).connectReqs.get(id);
        return req && req.promise ? req : undefined;
    };
    PendingRequests.prototype.getPing = function (streamId, id) {
        var req = this.getReqsByStreamId(streamId).pingReqs.get(id);
        return req && req.promise ? req : undefined;
    };
    PendingRequests.prototype.getCreatedDate = function (streamId, id) {
        var req = this.getReqsByStreamId(streamId).pingReqs.get(id);
        return req ? req.created : undefined;
    };
    PendingRequests.prototype.clean = function () {
        this.cleanAll(this.wcReqs.connectReqs);
        this.cleanAll(this.wcReqs.pingReqs);
        this.cleanAll(this.sigReqs.connectReqs);
        this.cleanAll(this.sigReqs.pingReqs);
    };
    PendingRequests.prototype.cleanAll = function (requests) {
        requests.forEach(function (req) { return req.reject(new Error('clean')); });
        requests.clear();
    };
    PendingRequests.prototype.getReqsByStreamId = function (streamId) {
        return streamId === this.wcStreamId ? this.wcReqs : this.sigReqs;
    };
    PendingRequests.prototype.addRequest = function (requests, id, timeout) {
        var req = { created: Date.now() };
        req.promise = new Promise(function (resolve, reject) {
            var timer = setTimeout(function () { return req.reject(new Error("Request " + timeout + "ms timeout")); }, timeout);
            var clean = function () {
                clearTimeout(timer);
                req.promise = undefined;
            };
            req.resolve = function () {
                clean();
                resolve();
            };
            req.reject = function (err) {
                if (req.promise) {
                    req.promise.catch(function () { });
                }
                clean();
                reject(err);
            };
            requests.set(id, req);
        });
        return req;
    };
    return PendingRequests;
}());
export { PendingRequests };
//# sourceMappingURL=PendingRequests.js.map