import { ReplaySubject } from 'rxjs';
import { log } from '../../misc/util';
var Remote = (function () {
    function Remote(id, pc, send, remotes) {
        var _this = this;
        this.peerToLog = 'INITIATOR';
        this.id = id;
        this.pc = pc;
        this.send = send;
        this._onError = function () { };
        this.candidates = new ReplaySubject();
        this.remotes = remotes;
        this.isSDPSent = false;
        this.remotes.set(id, this);
        pc.oniceconnectionstatechange = function () {
            _this.log('ICE CONNECTION STATE', pc.iceConnectionState);
            if (pc.iceConnectionState === 'failed') {
                _this._onError(new Error('Ice Connection State is FAILED'));
            }
        };
        pc.onicecandidate = function (evt) {
            if (evt.candidate !== null) {
                _this.log('LOCAL ICE CANDIDATE', evt.candidate.candidate);
                _this.send({
                    candidate: {
                        candidate: evt.candidate.candidate,
                        sdpMid: evt.candidate.sdpMid,
                        sdpMLineIndex: evt.candidate.sdpMLineIndex,
                    },
                });
            }
            else if (_this.isSDPSent) {
                _this.pc.onicecandidate = function () { };
                _this.log("FINISHED (onicecandidate) with: " + _this.id);
                _this.send(undefined);
            }
        };
    }
    Object.defineProperty(Remote.prototype, "onError", {
        get: function () {
            return this._onError;
        },
        set: function (handler) {
            this._onError = handler;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Remote.prototype, "onDataChannel", {
        set: function (handler) {
            var _this = this;
            this.pc.ondatachannel = function (_a) {
                var channel = _a.channel;
                _this.pc.oniceconnectionstatechange = function () { };
                handler(channel);
            };
        },
        enumerable: true,
        configurable: true
    });
    Remote.prototype.sdpIsSent = function () {
        this.isSDPSent = true;
        if (this.pc.iceGatheringState === 'complete') {
            this.pc.onicecandidate = function () { };
            this.log("FINISHED (sdpIsSent) with: " + this.id);
            this.send(undefined);
        }
    };
    Remote.prototype.clean = function () {
        this.pc.oniceconnectionstatechange = function () { };
        this.pc.onicecandidate = function () { };
        this.candidates.complete();
        this.remotes.delete(this.id);
        this.log("DELETE Remote: " + this.id);
        if (this.pc.iceConnectionState !== 'connected' && this.pc.iceConnectionState !== 'completed') {
            this.log('Failed to establish RTCDataChannel');
            this.pc.close();
            this.log("FINISHED (clean) with: " + this.id);
            this.send(undefined);
        }
    };
    Remote.prototype.handleMessage = function (msg) {
        var _this = this;
        if (msg.type) {
            switch (msg.type) {
                case 'offer':
                    this.log('REMOTE OFFER', { offer: msg.offer });
                    this.pc
                        .setRemoteDescription({ type: 'offer', sdp: msg.offer })
                        .then(function () {
                        return _this.candidates.subscribe(function (ic) {
                            return _this.pc
                                .addIceCandidate(ic)
                                .catch(function (err) { return _this.log('Failed to addIceCandidate', err); });
                        });
                    })
                        .then(function () { return _this.pc.createAnswer(); })
                        .then(function (answer) { return _this.pc.setLocalDescription(answer); })
                        .then(function () {
                        var sdp = _this.pc.localDescription.sdp;
                        _this.log('Send LOCAL ANSWER', { answer: sdp });
                        _this.send({ answer: sdp });
                        _this.sdpIsSent();
                    })
                        .catch(function (err) { return _this._onError(err); });
                    break;
                case 'answer':
                    this.log('REMOTE ANSWER is received', { answer: msg.answer });
                    this.pc
                        .setRemoteDescription({ type: 'answer', sdp: msg.answer })
                        .then(function () {
                        _this.candidates.subscribe(function (ic) {
                            return _this.pc
                                .addIceCandidate(ic)
                                .catch(function (err) { return _this.log(_this.id + ": Failed to add REMOTE Ice Candidate", err); });
                        });
                    })
                        .catch(function (err) { return _this._onError(err); });
                    break;
                case 'candidate':
                    this.log('REMOTE ICE CANDIDATE is received', msg.candidate);
                    this.candidates.next(new global.RTCIceCandidate(msg.candidate));
                    break;
                default:
                    this._onError(new Error('Buffer Protocol unknown message from the remote peer'));
            }
        }
        else {
            this.log('REMOTE peer FINISHED');
            this.candidates.complete();
            this.remotes.delete(this.id);
        }
    };
    Remote.prototype.log = function (msg, obj) {
        if (obj) {
            log.webrtc(this.peerToLog + ": " + msg, obj);
        }
        else {
            log.webrtc(this.peerToLog + ": " + msg);
        }
    };
    return Remote;
}());
export { Remote };
//# sourceMappingURL=Remote.js.map